#ifndef AVIFILE_SUBTITLE_H
#define AVIFILE_SUBTITLE_H

/** Maximum supported number of lines which may compose one subtitle */
#define SUBTITLE_MAX_LINES 5

/**
 * subtitle is plain C library - usable outside of avifile
 * once I had a dream we all could use such library to parse
 * various subtitle file - I've been naive...
 */

#if defined(__cplusplus)
extern "C" {
#endif

/**
 * Types of recognized subtitles
 * with a short example of such subtitle line
 */
typedef enum
{
    /** \internal */
    SUBTITLE_UNSELECTED,
    /** {123}{345} text */
    SUBTITLE_MICRODVD,
    /**
     * 1:23:55.125,1:24:00.275
     * text
     */
    SUBTITLE_SUBRIP,
    /**
     * 1:23:55
     * text
     */
    SUBTITLE_VPLAYER,
    SUBTITLE_AQT,
    SUBTITLE_SAMI,
    SUBTITLE_SUBVIEVER,
    SUBTITLE_MPSUB,		/* say hello to Gabuciono */
} subtitle_type;

typedef struct
{
    /** how many lines are in this subtitle */
    int lines;
    /** starting time for display  (time in 1/1000s = 1ms) */
    unsigned int start;
    /** ending time for display  (time in 1/1000s = 1ms) */
    unsigned int end;
    /** pointers to individual subtitle lines */
    char* line[SUBTITLE_MAX_LINES];
} subtitle_line_t;

typedef struct
{
    /* should not be accessed directly
     * - use suplied functions to manipulate with fields */
    char* filename;
    subtitle_type type;
    int allocated;
    int count;
    int frame_based; /* if subtitles use frame number */
    int time_diff;
    double fps;
    /* mutex for write will be added */
    void* mutex; // prevents us to use const (no mutable)
    subtitle_line_t* subtitle;
} subtitles_t;

/**
 * Opens subtiles from a given filename
 *
 * \param subset  subtitle array to be filled with subtitles.
 *   If subset is NULL than new set is created. Otherwise subset
 *   is just updated
 * \param filename read subtitles from the filename file
 */
subtitles_t* subtitle_open(subtitles_t* subs, const char* filename);
/**
 * Closes and destroys array of subtitles
 * \param subset  array of subtitles for freeing
 *
 * \warning after calling this function the subset array could no longer
 * be accessed as all the memory allocated by this structure has been
 * released
 */
void subtitle_close(subtitles_t* subs);
void subtitle_set_fps(subtitles_t*, double fps);
/** Returns number of lines in the subtitle array */
int subtitle_get_lines(subtitles_t*);
/** Returns type of subtitles */
subtitle_type subtitle_get_type(subtitles_t*);
/**
 * Copies subtitle into given subtitle_line_t pointer
 * It will check first if the line isn't already copied
 * returns true if subtitle is found
 */
int subtitle_get(subtitle_line_t*, subtitles_t*, double timepos);
/**
 * Check if two subtitles are equal
 * checking linecount & start,end pos (text is not being compared!)
 */
int subtitle_line_equals(const subtitle_line_t*, const subtitle_line_t*);
/** creates new empty subtitle line */
subtitle_line_t* subtitle_line_new(void);
/** duplicates one subtitle line */
subtitle_line_t* subtitle_line_copy(const subtitle_line_t*);
/** free allocated subtitle line (obtained with subtitle_line_copy) */
void subtitle_line_free(subtitle_line_t*);

/*
 will be implemented later

 write
 update
 insert
 delete
 */

#if defined(__cplusplus)
}
#endif

#endif
